#! /usr/bin/env python
# -*- coding: utf-8 -*-

'''
The script will move DaVinci Resolve DRP files from the watched directory
to the backup directory contained within and add a timestamp to the files.
The use case is where a Resolve user repeatedly saves DRP files on the fly
using the keyboard shortcut.

Set the path to the directory and the checking period interval in the 
user configuration section below. The path example will form this
POSIX path: /home/igor/watched in Linux and macOS. In Windows the path
example will create: C:\home\igor\watched.

The script will run with either Python 2.x or 3.x.


MIT License

Copyright (c) 2020 Igor Riđanović, metafide.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. 
'''


#--- User Configuration-----------------------------------

drpDirectory = os.path.join(os.sep, 'home','igor','watched')
interval     = 1   # Check for new DRP file every 45 seconds

#---- End user configuration------------------------------



import os
import sys
import time
from datetime import datetime


def console_print(txt):
	print(txt)


def get_files():

	# Make a list of all old files in the directory.
	dd = drpDirectory
	files = [f for f in os.listdir(dd) if os.path.isfile(os.path.join(dd, f))]

	return files


def setup():

	# Exit if the watched folder is missing.
	if not os.path.isdir(drpDirectory):
		sys.exit('The user set directory does not exist.')

	# Create the backup directory if missing.
	# We assume that the user has the write permission.
	backupDirectory = os.path.join(drpDirectory, 'DRP_Backups')

	if not os.path.isdir(backupDirectory):
		os.mkdir(backupDirectory)
		console_print('Created directory ' + backupDirectory)

	# Make a list of all old files in the directory.
	oldFiles = get_files()
	
	# Delete old files. Be careful!!!
	for f in oldFiles:
		fileDel = os.path.join(drpDirectory, f)
		os.remove(fileDel)
		console_print('Deleted: ' + fileDel)

	# We'll need the backup directory name in the main loop.
	return backupDirectory


def drp_is_saving(f):
	# OS portable method of determining when Resolve has stopped
	# saving the DRP. We watch for the file size to stop growing.

	while True:

		sizeA = os.path.getsize(f)
		time.sleep(5) # Pause before taking the 2nd measurement.
		sizeB = os.path.getsize(f)
	
		if sizeA == sizeB:
			break
		
		# Slow down the loop to prevent overloading.
		time.sleep(5)	


def main_loop():

	# Cleanup and setup directories. Also get the backup dir name.
	backupDirectory = setup()

	# Infinite loop.
	while True:

		# Make a list of all files in the directory. There should be only one file.
		drpFile = get_files()

		# There is DRP file. Combine the file name with the path.
		if len(drpFile) > 0:
			console_print('Found DRP file: ' + drpFile[0])
			drpFile = os.path.join(drpDirectory, drpFile[0])

			# Check if Resolve is saving.
			# This will block the loop until the DRP is fully saved.
			drp_is_saving(drpFile)

			# The DRP is finished saving. Let's add timestamp to it.
			stamp      = datetime.now().strftime('_%d-%b-%Y_%H:%M:%S')
			justName   =  os.path.basename(drpFile)
			drpRenamed = justName[:-4] + stamp + '.drp'
			drpRenamed = os.path.join(backupDirectory, drpRenamed)

			# Let's move the DRP to the backup directory.
			os.rename(drpFile, drpRenamed)

			console_print('Moved: ' + drpFile)
			console_print('To: '    + drpRenamed)

		console_print('Waiting...')
		time.sleep(interval)

	

if __name__ == '__main__':
	main_loop()

